#!/usr/bin/php
<?php

// Copyright (C) 2023 Cygnus <cygnus2 at post.cz>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or (at
// your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA

// BASH colors
define ("CFAIL","\033[1;31;40m !\033[0m ");
define ("COK","\033[1;32;40m *\033[0m ");
define ("CWARN","\033[1;33;40m ?\033[0m ");
define ("CTRY","\033[1;34;40m -\033[0m ");

define ("CRED","\033[1;31;40m");
define ("CGRN","\033[1;32;40m");
define ("CYELL","\033[1;33;40m");
define ("CBLUE","\033[1;34;40m");
define ("CMAGN","\033[1;35;40m");
define ("CCYAN","\033[1;36;40m");
define ("CWHITE","\033[1;37;40m");

define ("CDEF","\033[0m");

$debug = 1;
$comments = 1;			// basically it is more verbosity in generated source code
$cfg_xoffset = 11;		// character width (for monospace characters)

$scale = 3;			// must be > 1 otherwise dots will be converted wrong, 3 is optimum

$cfg_hex_caps = 0;

// --------------------------------------------------------------------------------------
function to_hex ($b)
{
	global $cfg_hex_caps;

	if ($cfg_hex_caps)
		$hextab = array ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
	else
		$hextab = array ('0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f');

	$h = floor($b / 16);
	$l = $b - ($h * 16);

	return ($hextab[$h].$hextab[$l]);
}

function convert ($infile, $filename)
{
	global $debug;
	global $comments;
	global $cfg_xoffset;
	global $scale;

	$pen = 0;	// 0 = pen up, 1 = pen down
	$char = 31;	// first ASCII code (for some reason space is twice in binary)
	
	$offsetx = -2;
	$offsety = -3;
	
	$glyph = array ();
	$pairs = 0;
	
	while (1)
	{
		$ch = fread($infile, 1);	// read one character from input file

		// check for end of file
		if ($ch === false || feof($infile))
		{
			break;
		}

		$b = unpack('C', $ch);		// unpack as unsigned char = byte (returns array)
		
		// bit 7 = end of sequence	url	https://oldcomp.cz/viewtopic.php?f=144&t=11890
		// bit 6 = pen dw/up
		// bit 3..5 = X	
		// bit 0..2 = Y
		
		/*if ($debug == 2)
		{
			echo to_hex($b[1])." ";
			if ($b[1] & 0b10000000) echo "\n";
		}*/
		
		/*if ($debug == 3)
		{
			if ($char < 128) $s = chr($char); else $s = ' ';
			echo "// ".$char." ".$s." ".$b[1]." ";
			echo "x=".(($b[1] & 0b00111000) >> 3)." ";
			echo "y=".($b[1] & 0b00000111)." ";
			if ($b[1] & 0b01000000) echo "b6 ";
			if ($b[1] & 0b10000000) echo "b7 ";
			echo "\n";
		}*/
		
		if ($pairs == 0)
		{
			// test bit 6 in first byte of sequence
			if ($b[1] & 0b01000000)
			{
				// with pen up move to the "start" coordinate
				$glyph[$pairs] = array (ord('R') + (0 + $offsetx)*$scale, ord('R') - (1 + $offsety)*$scale);
				$pairs++;
				// with pen down move to the coordinate from sequence
				$glyph[$pairs] = array (ord('R') + $scale*($offsetx + ($b[1] & 0b00000111)), ord('R') - $scale*($offsety + (($b[1] & 0b00111000) >> 3)));
				$pairs++;
			}
			else
			{
				// with pen up move to the coordinate from sequence
				$glyph[$pairs] = array (ord('R') + $scale*($offsetx + ($b[1] & 0b00000111)), ord('R') - $scale*($offsety + (($b[1] & 0b00111000) >> 3)));
				$pairs++;
			}
		}
		else
		{
			// add coordinates in array
			if ($b[1] & 0b01000000)
			{
				// move pen down if wasn't - or continue with pen down move to the coordinate from sequence
				$glyph[$pairs] = array (ord('R') + $scale*($offsetx + ($b[1] & 0b00000111)), ord('R') - $scale*($offsety + (($b[1] & 0b00111000) >> 3)));
				$pairs++;
			}
			else
			{
				// move pen up, then move to the coordinate from sequence
				$glyph[$pairs] = array (ord(' '), ord('R'));
				$pairs++;
				$glyph[$pairs] = array (ord('R') + $scale*($offsetx + ($b[1] & 0b00000111)), ord('R') - $scale*($offsety + (($b[1] & 0b00111000) >> 3)));
				$pairs++;
			}
		}
		
		// if bit 7 set finish glyph
		if ($b[1] & 0b10000000)
		{
			if ($char < 33)
			{
				if ($char == 32)
					echo "  699  1JZ\n";
			}
			else
			{
				// here is good place to check for dots where pen is not moved
				$newn = 0;
				$newpairs = $pairs;
				for ($n = 0; $n < $pairs; $n++)
				{
					if ($n > 0 && ($glyph[$n-1][0] == $glyph[$n][0]) && ($glyph[$n-1][1] == $glyph[$n][1]))
					{
						// dot detected -> make it bigger
						$newglyph[$newn][0] = $glyph[$n][0]-1;
						$newglyph[$newn][1] = $glyph[$n][1]-1;
						$newn++;
						$newpairs++;
						$newglyph[$newn][0] = $glyph[$n][0];
						$newglyph[$newn][1] = $glyph[$n][1]-2;
						$newn++;
						$newpairs++;
						$newglyph[$newn][0] = $glyph[$n][0]+1;
						$newglyph[$newn][1] = $glyph[$n][1]-1;
						$newn++;
						$newpairs++;
						$newglyph[$newn][0] = $glyph[$n][0];
						$newglyph[$newn][1] = $glyph[$n][1];
						$newn++;
					}
					else
					{
						// copy pair
						$newglyph[$newn][0] = $glyph[$n][0];
						$newglyph[$newn][1] = $glyph[$n][1];
						$newn++;
					}
				}
				
				$glyph = $newglyph;
				$pairs = $newpairs++;
				
				// glyph identificator in Hershey's catalog
				$s = "12345";
				
				// add one pair for limits
				$pairs++;
				
				// number of pairs
				if ($pairs < 10) $s = $s.' ';
				if ($pairs < 100) $s = $s.' ';
				$s .= $pairs;
				
				// limits - always same, original font is monospace
				$s .= chr($scale*$offsetx+ord('R'));
				$s .= chr($scale*($offsetx+5)+ord('R'));
				
				// subtract one pair for limits, then add all pairs in string
				for ($n = 0; $n < ($pairs - 1); $n++)
				{
					$s .= chr($glyph[$n][0]);
					$s .= chr($glyph[$n][1]);
// 					$s .= ' ';
					
				}
				
				echo $s."\n";
			}

			$s = "";
			$pairs = 0;
			$char++;
		}
	}
	
	while ($char < 128)
	{
		// add space in case there was less characters than full ASCII set
		echo "  699  1JZ\n";
	}
}

// --------------------------------------------------------------------------------------

if (isset ($argv[1]))
{
	$option_count=0;
	foreach ($argv as $key => $option)
	{
		// ignore $option_count[0], because it is filename of script itself
		if (($option_count > 0) && ($option[0] != '-'))
		{
			if (file_exists ($option))
			{
				$in = fopen ($option, "r");

				if ($in)
				{
					convert ($in, $option);
				}
				else
				{
					if (!$in) echo CFAIL."Opening input file ".$option." failed.\n".CDEF;
				}
				if ($in) fclose ($in);
			}
			else
			{
				echo CFAIL."File ".$option." not found.\n".CDEF;
			}
		}
		$option_count++;
	}
}
else
	echo CFAIL."You must specify at least one filename for input file.\n".CDEF;

return (0);

?>
\
