#!/usr/bin/php
<?php

// Copyright (C) 2023 Cygnus <cygnus2 at post.cz>
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or (at
// your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA

// BASH colors
define ("CFAIL","\033[1;31;40m !\033[0m ");
define ("COK","\033[1;32;40m *\033[0m ");
define ("CWARN","\033[1;33;40m ?\033[0m ");
define ("CTRY","\033[1;34;40m -\033[0m ");

define ("CRED","\033[1;31;40m");
define ("CGRN","\033[1;32;40m");
define ("CYELL","\033[1;33;40m");
define ("CBLUE","\033[1;34;40m");
define ("CMAGN","\033[1;35;40m");
define ("CCYAN","\033[1;36;40m");
define ("CWHITE","\033[1;37;40m");

define ("CDEF","\033[0m");

$comments = 1;		// basically it is more verbosity in generated source code
$cfg_xoffset = 3;	// o kolik vpravo - character width (for monospace characters) default 11, rowmans 0, cursive 8
			// mzxr +3
$cfg_yoffset = 11;	// o kolik nahoru (default 16, cursive 21)
			// mzxr +11
$cfg_monospace = 0;	// 1 = enable monospace

$set_iso88592 = [
		' ', 'Ą', '˘', 'Ł', '¤', 'Ľ', 'Ś', '§', '¨', 'Š', 'Ş', 'Ť', 'Ź', ' ', 'Ž', 'Ż',
		'°', 'ą', '˛', 'ł', '´', 'ľ', 'ś', 'ˇ', '¸', 'š', 'ş', 'ť', 'ź', '˝', 'ž', 'ż',
		'Ŕ', 'Á', 'Â', 'Ă', 'Ä', 'Ĺ', 'Ć', 'Ç', 'Č', 'É', 'Ę', 'Ë', 'Ě', 'Í', 'Î', 'Ď',
		'Đ', 'Ń', 'Ň', 'Ó', 'Ô', 'Ő', 'Ö', '×', 'Ř', 'Ů', 'Ú', 'Ű', 'Ü', 'Ý', 'Ţ', 'ß',
		'ŕ', 'á', 'â', 'ă', 'ä', 'ĺ', 'ć', 'ç', 'č', 'é', 'ę', 'ë', 'ě', 'í', 'î', 'ď',
		'đ', 'ń', 'ň', 'ó', 'ô', 'ő', 'ö', '÷', 'ř', 'ů', 'ú', 'ű', 'ü', 'ý', 'ţ', '˙'
		];

// --------------------------------------------------------------------------------------

function convert ($infile, $filename)
{
	global $comments;
	global $cfg_xoffset;
	global $cfg_yoffset;
	global $cfg_monospace;
	global $set_iso88592;

	$pen = 0;	// 0 = pen up, 1 = pen down
	$char = 32;	// first ASCII code
	
	// start of PHP
	echo "<?php\n";
	
	// start of array
	if ($comments) echo "// PHP array - font ".$filename."\n";
	echo "\$font = [\n";
	
	$maxw = 0;	// minimal width
	$minw = 1000;	// maximal width
	$minx = 1000;
	$maxx = 0;
	$miny = 1000;
	$maxy = 0;

	while (($line = fgets($infile)) !== false) 
	{
		$length = strlen($line);
		if ($length > 8)
		{
			$glyph_number = 0;
			
			// 5 bytes = character number in Hershey font
			for ($n = 0; $n < 5; $n++)
			{
				if (($line[$n] >= '0') && ($line[$n] <= '9'))
				{
					$glyph_number = $glyph_number * 10;
					$glyph_number += ($line[$n] - '0');
				}
			}
			
			// 3 bytes =  length of character in pairs of coordinates
			$pairs = 0;
			for ($n = 5; $n < 8; $n++)
			{
				if ((ord($line[$n]) > 47) && (ord($line[$n]) < 58))
				{
					$pairs = $pairs * 10;
					$pairs += ($line[$n] - '0');
// 					echo CRED.$line[$n].CDEF;
				}
			}

			// whole original encoding in comment if commens are enabled
			if ($comments)
			{
				// ASCII without control codes
				if ($char > 31 && $char < 127)
					echo "\t// char. ".chr($char)." (".$char.") glyph ".$glyph_number." ";
					
				// between ASCII and extension
				if ($char > 126 && $char < 160)
					echo "\t// char. ".chr(32)." (".$char.") glyph ".$glyph_number." ";
					
				// extension
				if ($char > 159 && $char < 256)
					echo "\t// char. ".$set_iso88592[$char-160]." (".$char.") glyph ".$glyph_number." ";
				
				
				for ($n = 0; $n < $pairs; $n++)
				{
					// every coordinate is exactly one byte, adding space between them to be more readable 
					echo ($line[8+2*$n].$line[9+2*$n])." ";
				}
				echo "\n"; 
			}
			
			// minx=".$minx." maxx=".$minx." miny=".$miny." maxy=".$miny." "
			
			// start of line in array
			echo "\t".$char." => [".($pairs-1).", ";
			
			$coordinate_counter = 0;
			
			for ($n = 0; $n < $pairs; $n++)
			{
				if ($n == 0)
				{
					// pair 0 defines width of glyph
					$width = ord($line[9+2*$n]) - ord($line[8+2*$n]);
					
					$leftmargin = ord('R') - ord($line[8+2*$n]);
					$rightmargin = -ord('R') + ord($line[9+2*$n]);
					
// 					echo " *L ".$leftmargin." ";
// 					echo " *R ".$rightmargin." "; 
					
					echo $width.", ";
					
					if ($maxw < $width) $maxw = $width;
					if ($minw > $width) $minw = $width;
				}
				
				if ($n > 0)
				{
					if (($line[8+2*$n] == ' ') && ($line[9+2*$n] == 'R'))
					{
						// pair " R" means pen UP
						// all coordinates will be in separate included array
						if ($coordinate_counter > 0) 
							echo ", ";		// separate coordinates

						echo "[-1,-1]";		// equivalent for pen UP
						$pen = 0;
					} else {
						// any other pair means pen DOWN and move to coordinates
						$x = -(ord('R') - ord($line[8+2*$n]));
						$y = ord('R') - ord($line[9+2*$n]);
						
						// replace with constant if monospace
						if ($cfg_monospace)
							$x = $cfg_xoffset + $x;
						else
							$x = $leftmargin + $x;

						$x = $cfg_xoffset + $x;
						$y = $cfg_yoffset + $y;
						
						// for info in comments
						if ($n > 0)
						{
							if ($maxx < $x) $maxx = $x;
							if ($maxy < $y) $maxy = $y;
							if ($minx > $x) $minx = $x;
							if ($miny > $y) $miny = $y;
						}
						
						// all coordinates will be in separate included array
						if ($coordinate_counter > 0) 
							echo ", ";		// separate coordinates

						echo "[".$x.",".$y."]";		// coordinates - small array with two items
						
						$pen = 1;
						$coordinate_counter++;
					}
				}
			}
			
			// add dummy coordinates if character is empty (space for example - but it still has width)
			if ($pairs == 1)
				echo "[-1,-1]";	// pen up - special characters without pairs
			
			echo "],\n";		// close array of one character
			$pen = 0;
			$char++;
		}
	}
	echo "];\n";	// close array of whole font
	
	echo "// max width = ".$maxw."\n";
	echo "// min width = ".$minw."\n";
	echo "// max x = ".$maxx."\n";
	echo "// min x = ".$minx."\n";
	echo "// max y = ".$maxy."\n";
	echo "// min y = ".$miny."\n";
	echo "// last character char = ".$char."\n";
	
	echo "?>\n";	// close PHP
}

// --------------------------------------------------------------------------------------

if (isset ($argv[1]))
{
	$option_count=0;
	foreach ($argv as $key => $option)
	{
// 		echo $key."\n";
		// ignore $option_count[0], because it is filename of script itself
		if (($option_count > 0) && ($option[0] != '-'))
		{
			if (file_exists ($option))
			{
				$in = fopen ($option, "r");

				if ($in)
				{
					// echo CYELL."Opening filename: ".$option."\n".CDEF;
					convert ($in, $option);
				}
				else
				{
					if (!$in) echo CFAIL."Opening input file ".$option." failed.\n".CDEF;
				}
				if ($in) fclose ($in);
			}
			else
			{
				echo CFAIL."File ".$option." not found.\n".CDEF;
			}
		}
		$option_count++;
	}

}
else
	echo CFAIL."You must specify at least one filename for input file.\n".CDEF;

return (0);

?>
