#!/usr/bin/python

# Copyright (C) 2023 Cygnus <cygnus2 at post.cz>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import os
os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide"
import pygame
import sys, getopt

pygame.display.set_caption('A.V.Heshey character viewer')
pygame.font.init()
font = pygame.font.get_default_font()
fontsmall = pygame.font.SysFont(font, 24)
fontbig = pygame.font.SysFont(font, 32)

debug = 2			# defacto verbosity, can be 0,1,2

w = 1280
h = 1024

# bash / ANSI colors
bcol_def = "\033[0m"		# default color (text reset)
bcol_red = "\033[0;91m"
bcol_green = "\033[0;92m"
bcol_blue = "\033[0;94m"
bcol_yellow = "\033[0;93m"
bcol_purple = "\033[0;95m"
bcol_cyan = "\033[0;96m"
bcol_white = "\033[0;97m"	# high intensity white

# debug color test
#print (bcol_red+"red"+bcol_def)
#print (bcol_green+"green"+bcol_def)
#print (bcol_blue+"blue"+bcol_def)
#print (bcol_yellow+"yellow"+bcol_def)
#print (bcol_purple+"purple"+bcol_def)
#print (bcol_cyan+"cyan"+bcol_def)
#print (bcol_white+"white"+bcol_def)

color_background = (200,200,200)	# light gray
color_lightbackground = (220,220,220)	# brighter gray
color_grey = (150,150,150)		# dark gray
color_template = (255,255,255)		# white
color_selection = (0,220,220)		# light cyan
color_raster = (0,128,128)		# dark cyan
color_guideline = (0,128,255)		# greenish blue
color_limits = (255,0,0)		# red
color_line = (0,0,0)			# black
color_text = (0,0,0)			# black

rasterxpos = 20			# position of raster
rasterypos = 20
rastersize = 43			# number of squares (columns and rows)
rastersquare = 22

lineh = 20			# line height for text at right side of window

screen = pygame.display.set_mode((w, h))
screen.fill(color_background)

hershey_font = []
hershey_space = " 2749  1JZ"
clipboard = hershey_space

maxchar = 32			# last character 127 if no extension included
				# last character 255 if extension enabled for ISO8859-2
extensionset = 0		#  0 = standard ASCII, 1 = ISO8859-2


# guide line array should contain 0 = no line, 1 = line
guidelnx = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]	# guide lines x
guidelny = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]	# guide lines y

# conversion table iso-8859-2 -> utf-8 (codes 0xAx..0xFF // 160..255)
set_iso88592 = [' ', 'Ą', '˘', 'Ł', '¤', 'Ľ', 'Ś', '§', '¨', 'Š', 'Ş', 'Ť', 'Ź', ' ', 'Ž', 'Ż', 
		'°', 'ą', '˛', 'ł', '´', 'ľ', 'ś', 'ˇ', '¸', 'š', 'ş', 'ť', 'ź', '˝', 'ž', 'ż',
		'Ŕ', 'Á', 'Â', 'Ă', 'Ä', 'Ĺ', 'Ć', 'Ç', 'Č', 'É', 'Ę', 'Ë', 'Ě', 'Í', 'Î', 'Ď',
		'Đ', 'Ń', 'Ň', 'Ó', 'Ô', 'Ő', 'Ö', '×', 'Ř', 'Ů', 'Ú', 'Ű', 'Ü', 'Ý', 'Ţ', 'ß',
		'ŕ', 'á', 'â', 'ă', 'ä', 'ĺ', 'ć', 'ç', 'č', 'é', 'ę', 'ë', 'ě', 'í', 'î', 'ď',
		'đ', 'ń', 'ň', 'ó', 'ô', 'ő', 'ö', '÷', 'ř', 'ů', 'ú', 'ű', 'ü', 'ý', 'ţ', '˙']

def wait_for_key ():
	x = -1		# coordinates outside screen = wasn't clicked
	y = -1

	while True:
		for events in pygame.event.get():
			if events.type == pygame.KEYDOWN:
				return [x,y]
			if events.type == pygame.MOUSEBUTTONDOWN:
				pos = pygame.mouse.get_pos()
				x = pos[0]
				y = pos[1]
				return [x,y]

		pygame.time.wait(2)
	
#---------------------------------------------------------------------------------------------------

def helppage ():
	screen.fill(color_background)
	
	x = 100
	y = lineh
	
	msg = fontsmall.render("Help", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+2*lineh
	msg = fontsmall.render("F1 = this help page", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("F5 = refresh", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+2*lineh
	msg = fontsmall.render("A = whole ASCII with possible selection", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("B = toggle bold/regular display of character", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("R = slow redraw", 1, color_text)
	screen.blit(msg, (x,y))

	y = y+lineh
	msg = fontsmall.render("H = Horizontal flip", 1, color_text)
	screen.blit(msg, (x,y))

	y = y+lineh
	msg = fontsmall.render("V = Vertical flip", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("P = toggle between coordinates and pen up", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("E = export character, will print whole Hershey character in console", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("I = info about font", 1, color_text)
	screen.blit(msg, (x,y))

	y = y+lineh
	msg = fontsmall.render("X = toggle ASCII & font extension = ISO8859-2 page or expand ASCII up to 255 characters", 1, color_text)
	screen.blit(msg, (x,y))

	y = y+lineh
	msg = fontsmall.render("cursors = move selected node (pair. but with exception for pen up)", 1, color_text)
	screen.blit(msg, (x,y))

	y = y+lineh
	msg = fontsmall.render("numpad cursors = move whole glyph to all directions", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("INSERT = add pair of coordinates (default RR)", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("DELETE = remove selected pair", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("PAGE UP / numpad + = move glyph selection", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("PAGE DOWN / numpad - = move glyph selection", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("HOME = move pair selection to the left", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("END = move pair selection to the right", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("CTRL + C = copy glyph in clipboard", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("CTRL + V = copy glyph from clipboard if there is any", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("CTRL + M = merge glyph - this will add glyph from clipboard to the selected glyph", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("CTRL + E = export whole font in console", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("CTRL + S = export whole font in console without additional comments (SAVE)", 1, color_text)
	screen.blit(msg, (x,y))
	
	
	y = y+lineh*2
	msg = fontsmall.render("Alt+F4 = Quit (only in main screen)", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh*2
	msg = fontsmall.render("written by ZX Cygnus 2023", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("https://cygnus.speccy.cz", 1, color_text)
	screen.blit(msg, (x,y))
	
	pygame.display.flip()
	wait_for_key()
	
#---------------------------------------------------------------------------------------------------
# only option how to "save" data - originally this program shouldn't be editor, just viewer

def export_character (character, extension = 0, maxchar = 127):
	if (maxchar == 127):
		if (character > 31) and (character < 128):
			print (bcol_green+"ASCII: "+str(character)+" ["+chr(character)+"] / character ISO8859-2: "+str(character+128)+" ["+set_iso88592[character-32]+"]"+bcol_def)
			print (hershey_font[character])
		else:
			print (bcol_red+"wrong character code (only ascii)"+str(character)+bcol_def)
	else:
		if (extension == 0):
			if (character > 31) and (character < 128):
				print (bcol_green+"ASCII: "+str(character)+" ["+chr(character)+"]"+bcol_yellow+" ln "+str(character-31)+bcol_def)
				print (hershey_font[character])
			else:
				print (bcol_red+"wrong character code (extended ascii) "+str(character)+bcol_def)
		else:
			if (character > 159) and (character < 256):
				print (bcol_green+"ISO8859-2: "+str(character)+" ["+set_iso88592[character-160]+"]"+bcol_yellow+" ln "+str(character-31)+bcol_def)
				print (hershey_font[character])
			else:
				print (bcol_red+"wrong character code (extension set) "+str(character)+bcol_def)
		

#---------------------------------------------------------------------------------------------------
# draw one centered glyph

def character_on_xy (chx, chy, character, scale = 2, frame=0):
	# xy is center of character, coordinates [R,R]
	hchar = hershey_font[character]
	pairs = hershey_pairs(hchar)	# get number of pairs

	# left and right limit
	limitleft = ord('R') - ord(hchar[8])
	limitright = -(ord('R') - ord(hchar[9]))
	
	# 20 is position of center line R
	lny = chy+scale*20
	lnx = chx+scale*(20-limitleft)
	pygame.draw.line(screen,color_raster, (lnx, lny-scale), (lnx, lny+scale), 1)
	
	lny = chy+scale*20
	lnx = chx+scale*(20+limitright)
	pygame.draw.line(screen,color_raster, (lnx, lny-scale), (lnx, lny+scale), 1)
	
	if frame==1:
		pygame.draw.line(screen,color_grey, (chx, chy), (chx+40*scale, chy), 1)
		pygame.draw.line(screen,color_grey, (chx, chy), (chx, chy+40*scale), 1)
		pygame.draw.line(screen,color_grey, (chx, chy+40*scale), (chx+40*scale, chy+40*scale), 1)
		pygame.draw.line(screen,color_grey, (chx+40*scale, chy), (chx+40*scale, chy+40*scale), 1)
	
	pen = 0
	lastx = 0
	lasty = 0
	
	for i in range(1,pairs):
		x = ord(hchar[2*i+8]) - ord('R')
		y = ord(hchar[2*i+9]) - ord('R')
		
		if hchar[2*i+8]+hchar[2*i+9] == " R":
			pen = 0
		else:
			if pen == 0:
				lastx = chx+scale*(20+x)
				lasty = chy+scale*(20+y)
				pen = 1
			else:
				# 20 is position of center line R
				lnx = chx+scale*(20+x)
				lny = chy+scale*(20+y)
				pygame.draw.line(screen, color_line, (lastx, lasty), (lnx, lny), 1)
				lastx = lnx
				lasty = lny

#---------------------------------------------------------------------------------------------------
# one page of characters 32..127 or 160..255 if extension set enabled

def print_grid_of_characters (c = 32, character = 0):
	screen.fill (color_background)
	scale = 2

	# 8 lines, 16 columns
	for line in range(1,7):
		for column in range(0,16):
			character_on_xy (column*80, line*110, c, scale,1)
			
			if (c < 128):
				ch = chr(c)
			else:
				ch = set_iso88592[c-160]
			
			msg = fontsmall.render(str(c), 1, color_raster)
			screen.blit(msg, (column*80+4, line*110-20))
			
			msg = fontsmall.render(ch, 1, color_grey)
			screen.blit(msg, (column*80+4+40, line*110-20))
			
			c=c+1
			
	x = 50
	y = 8*110
	
	msg = fontsmall.render("click on glyph which you want view/edit", 1, color_text)
	screen.blit(msg, (x,y))

	y = y+lineh
	msg = fontsmall.render("A = return back without selecting another glyph", 1, color_text)
	screen.blit(msg, (x,y))
	
	y = y+lineh
	msg = fontsmall.render("X = toggle ASCII & font extension (ISO8859-2) if extension exists", 1, color_text)
	screen.blit(msg, (x,y))

#---------------------------------------------------------------------------------------------------
# character selection plus switching between pages

def glyph_selection (extension = 0, maxchar = 127):
	if (maxchar == 127):
		c = 32
	else:
		if (extension == 0):
			c = 32
		else:
			c = 160;
	
	r = -1		# return value
	
	print_grid_of_characters (c)
	pygame.display.flip()
	
	clickx = -1
	clicky = -1
	exit = 0
	
	while True:
		for events in pygame.event.get():
			if events.type == pygame.VIDEOEXPOSE:
				pygame.display.flip()
			
			if events.type == pygame.KEYDOWN:
				key = pygame.key.get_pressed()
				
				if key[pygame.K_a]:
					exit = 1
				
				if key[pygame.K_x]:
					if not (maxchar == 127):
						if (extension == 0):
							extension = 1
							c = 160
						else:
							extension = 0
							c = 32
						
						print_grid_of_characters (c)
						pygame.display.flip()
				
			if events.type == pygame.MOUSEBUTTONDOWN:
				pos = pygame.mouse.get_pos()
				clickx = pos[0]
				clicky = pos[1]
				print (clickx,clicky)
				exit = 1
			
			if events.type == pygame.QUIT:
				quit(0)
			
		if (exit == 1):
			break
		else:
			pygame.time.wait(2)
	
	# process redurned coordinates
	if (clickx != -1 and clicky != -1):
		print (bcol_white+"x =", clickx, "y =", clicky, bcol_def)
		chx = round((clickx-40) / 80)
		chy = round((clicky-145) / 110)
		if (chx > -1 and chy > -1):
			char = 32+16*chy+chx
			print (bcol_cyan,chx,",",chy,"=",char,bcol_def)
			if (char > 31 and char < 128):
				r = char
				
	if (maxchar == 255) and (extension == 1) and (r > -1):
		r = r +128
	
	return(r)

#---------------------------------------------------------------------------------------------------
# grid with coordinates around it

def basegrid (fontname = "", extension = 0):
	
	fontsize = "font size ASCII 32..127"
	if (extension == 1):
		fontsize = fontsize + " + ISO8859-2 extension 160..255"
	
	msg = fontsmall.render("Filename '"+fontname+"' - "+fontsize, 1, color_text)
	screen.blit(msg, (rasterxpos, 2))
	
	ch = ord('>')	# first Hershey coordinate
	linesize = [2,1,1,1,1,2,1,1,1,1,2,1,1,1,1,2,1,1,1,1,3,1,1,1,1,2,1,1,1,1,2,1,1,1,1,2,1,1,1,1,2,1]	# default line thicknesses
	
	size = rastersize
	lsp = 32		# space between grid and legend around it
	
	lny = rasterypos+(rastersquare-1)*size
	for x in range(0, size-1):
		lnx = rasterxpos+x*rastersquare
		if (guidelnx[x] == 0):
			pygame.draw.line(screen, color_raster, (lnx, rasterypos), (lnx, lny), linesize[x])
		else:
			pygame.draw.line(screen, color_guideline, (lnx, rasterypos), (lnx, lny), linesize[x]+1)
		msg = fontbig.render(chr(ch), 1, color_text)
		msgrect = msg.get_rect(center = (lnx,(rastersquare-1)*size+32))
		screen.blit(msg, msgrect)
		ch = ch+1
		
	ch = ord('>')	# first Hershey coordinate
	
	lnx = rasterxpos+(rastersquare-1)*size
	for y in range(0, size-1):
		lny = rasterypos+y*rastersquare
		if (guidelny[y] == 0):
			pygame.draw.line(screen, color_raster, (rasterxpos, lny), (lnx, lny), linesize[y])
		else:
			pygame.draw.line(screen, color_guideline, (rasterxpos, lny), (lnx, lny), linesize[y]+1)
		msg = fontbig.render(chr(ch), 1, color_text)
		msgrect = msg.get_rect(center = ((rastersquare-1)*size+32, lny))
		screen.blit(msg, msgrect)
		ch = ch+1

#---------------------------------------------------------------------------------------------------
# returns number of pairs - getting number of pairs from record in glyph data

def hershey_pairs (hchar):
	pairs = 0
	for i in range(5, 8):
		if (ord(hchar[i]) > 47 and ord(hchar[i]) < 58):
			pairs = pairs*10
			pairs = pairs+ord(hchar[i])-48

	if (debug > 2):
		print ("pairs: ",pairs)

	return(pairs)

#---------------------------------------------------------------------------------------------------
# additional statistics about font

def hershey_statistics ():
	x = 960
	y = rasterypos+10*lineh
	w = 300
	h = 6*lineh
	
	screen.fill(color_background, (x-4, y-4, w+1, h+1))
	
	pygame.draw.line(screen,color_grey, (x-4, y-4), (x-4+w, y-4), 1)
	pygame.draw.line(screen,color_grey, (x-4, y-4), (x-4, y-4+h), 1)
	pygame.draw.line(screen,color_grey, (x-4+w, y-4), (x-4+w, y-4+h), 1)
	pygame.draw.line(screen,color_grey, (x-4, y-4+h), (x-4+w, y-4+h), 1)
	
	msg = fontsmall.render("Info: ", 1, color_text)
	screen.blit(msg, (x,y+0*lineh))
	
	minpairs = 65535
	minchar = 0
	maxpairs = -1
	maxchar = 0
	pairstotal = 0
	
	for i in range (32, 127):
		pairs = hershey_pairs(hershey_font[i])
		pairstotal = pairstotal +pairs
		if (minpairs > pairs):
			minpairs = pairs
			minchar = i
		if (maxpairs < pairs):
			maxpairs = pairs
			maxchar = i
	
	if (debug):
		print ("min=",minpairs,minchar,"max=",maxpairs,maxchar,"avg=",pairstotal/96)
	
	msg = fontsmall.render("min pairs "+str(minpairs), 1, color_text)
	screen.blit(msg, (x,y+1*lineh))
	msg = fontsmall.render("chr("+str(minchar)+") = "+chr(minchar), 1, color_text)
	screen.blit(msg, (x+120,y+1*lineh))
	msg = fontsmall.render("max pairs "+str(maxpairs), 1, color_text)
	screen.blit(msg, (x,y+2*lineh))
	msg = fontsmall.render("chr("+str(maxchar)+") = "+chr(maxchar), 1, color_text)
	screen.blit(msg, (x+120,y+2*lineh))
	msg = fontsmall.render("average "+str(round(pairstotal/96))+" pairs", 1, color_text)
	screen.blit(msg, (x,y+3*lineh))

	
	# TODO - min/max width, highest/lowest coordinate, averages

#---------------------------------------------------------------------------------------------------
# list of hershey glyph coordinates under raster with glyph

def render_hershey_coordinates (hchar, selpair = 0):
	x = 12
	y = rasterypos+rastersquare*42+10

	xstep = 30
	ystep = 20
	
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	# erase background
	screen.fill(color_background, (x-4, y-2, 42*xstep, 3*ystep))
	
	# rectangles around letters
	for i in range(0,4):
		pygame.draw.line(screen,color_grey, (x-4, y-2+i*ystep), (x-4+42*xstep, y-2+i*ystep), 1)
	for i in range(0,43):
		pygame.draw.line(screen,color_grey, (x-4+i*xstep, y-2), (x-4+i*xstep, y-2+3*ystep), 1)
	
	# list of hershey coordinates
	for i in range(1,pairs):
		# highlight selected pair
		if (i == selpair):
			print ("highlighting",i)
			screen.fill(color_selection, (x-4, y-2, xstep, ystep))
		
		# print pair
		msg = fontsmall.render(hchar[2*i+8]+hchar[2*i+9], 1, color_text)
		screen.blit(msg, (x,y))
		
		# move to the next coordinates
		x = x + xstep
		if x > 1260:
			x = 12
			y = y + ystep
			
#---------------------------------------------------------------------------------------------------
# will return number of pair

def click_on_hershey_coordinates (hchar, clickpos):
	x = 12+12
	y = rasterypos+rastersquare*42+10+8

	xstep = 30	# 42 columns in 1280px
	ystep = 20	# 3 rows
	
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	column = round((clickpos[0] - x)/xstep)		# should be 0..41
	row = round((clickpos[1] - y)/ystep)		# should be 0..2
	
	if (debug > 1):
		print("row",row, "column",column)
		
	selpair = -1		# default return value
	
	if (column > -1) and (column < 42) and (row > -1) and (row < 3):
		# one pair is for left and right limit -> +1
		pair = row*42+column+1
		# return only existing pair in selected character
		if (pair < pairs):
			selpair = pair
	
	return(selpair)

#---------------------------------------------------------------------------------------------------
# returns 0 = not pen up, 1 = is pen up (" R")

def is_penup (hchar, selpair):
	if (selpair > 0):
		if (hchar[selpair*2+8] == ' ') and (hchar[selpair*2+9] == 'R'):
			return(1)
		else:
			return(0)
	else:
		return(0)

#---------------------------------------------------------------------------------------------------

def render_hershey_glyph (hchar, character, slow = 0, bold = 0, selpair = -1, extension = 0, maxchar = 127):
	x = 960
	y = rasterypos
	
	thickness = 8 + bold*8		# values 8, 16, 24
	
	if (maxchar == 127):
		glyphmsg = "ASCII: "+str(character)+" ["+chr(character)+"] ISO8859-2 ["+set_iso88592[character-32]+"]"
		line = character - 31
	else:
		if (extension == 0):
			glyphmsg = "ASCII: "+str(character)+" ["+chr(character)+"]"
			line = character - 31
		else:
			glyphmsg = "ISO8859-2: "+str(character)+" ["+set_iso88592[character-160]+"]"
			line = character - 31
	
	msg = fontbig.render(glyphmsg, 1, color_text)
	screen.blit(msg, (x,y))
	y = y+2*lineh
	
	msg = fontsmall.render("glyph: "+hchar[0:5]+" line: "+str(line), 1, color_text)
	screen.blit(msg, (x,y))
	y = y+lineh
	
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	msg = fontsmall.render("pairs with limits: "+str(pairs), 1, color_text)
	screen.blit(msg, (x,y))
	
	# left and right limit
	limitleft = ord('R') - ord(hchar[8])
	limitright = -(ord('R') - ord(hchar[9]))
	
	print ("limits ",limitleft,"-",limitright)
	
	# 20 is position of center line R
	lny = rasterypos+rastersquare*20
	lnx = rasterxpos+rastersquare*(20-limitleft)
	pygame.draw.line(screen,color_limits, (lnx, lny-rastersquare), (lnx, lny+rastersquare), 8)
	
	lny = rasterypos+rastersquare*20
	lnx = rasterxpos+rastersquare*(20+limitright)
	pygame.draw.line(screen,color_limits, (lnx, lny-rastersquare), (lnx, lny+rastersquare), 8)
	
	pen = 0
	lastx = 0
	lasty = 0
	
	for i in range(1,pairs):
		print (hchar[2*i+8]+hchar[2*i+9], end=' '),
		x = ord(hchar[2*i+8])-ord('R')
		y = ord(hchar[2*i+9])-ord('R')
		
		if hchar[2*i+8]+hchar[2*i+9] == " R":
			print ("[pen up]")
			pen = 0
		else:
			if pen == 0:
				lastx = rasterxpos+rastersquare*(20+x)
				lasty = rasterypos+rastersquare*(20+y)
				pen = 1
				#print ("[d]", end='')
			else:
				# 20 is position of center line R
				lnx = rasterxpos+rastersquare*(20+x)
				lny = rasterypos+rastersquare*(20+y)
				pygame.draw.line(screen, color_line, (lastx, lasty), (lnx, lny), thickness)
				lastx = lnx
				lasty = lny
				#print ("[-]", end='')
			#print (x,y)
		if slow == 1:
			# slow redraw
			pygame.time.delay(500)
			pygame.display.flip()
			
	if (selpair > -1):
		# draw circle on raster - nodes
		x = rasterxpos+rastersquare*(20+ord(hchar[2*selpair+8])-ord('R'))
		y = rasterypos+rastersquare*(20+ord(hchar[2*selpair+9])-ord('R'))
		pygame.draw.circle(screen, color_raster, (x, y), 6)
		
	if (selpair == -2) or (selpair == -3):
		# draw circle on raster - limits
		if (selpair == -2):
			x = rasterxpos+rastersquare*(20+ord(hchar[8])-ord('R'))
		if (selpair == -3):
			x = rasterxpos+rastersquare*(20+ord(hchar[9])-ord('R'))
		y = rasterypos+rastersquare*(20)
		pygame.draw.circle(screen, color_limits, (x, y), 6)

	print ("")

#---------------------------------------------------------------------------------------------------
# modify one pair of coordinates - move coordinates in any direction

def move_pair (hchar, pair, vector):
	# hershey_font[character], pair, (-1,0)
	x = ord(hchar[pair*2+8])+vector[0]
	y = ord(hchar[pair*2+9])+vector[1]
	
	# new string must be created because Python is not capable modify existing string
	# coordinates must be between '=' and 'h'
	if (x > 61) and (x < 104) and (y > 61) and (y < 104):
		newchar= ""
		for i in range(0,len(hchar)):
			if (i == pair*2+8) or (i == pair*2+9):
				if (i == pair*2+8):
					newchar = newchar + chr(ord(hchar[pair*2+8])+vector[0])
				if (i == pair*2+9):
					newchar = newchar + chr(ord(hchar[pair*2+9])+vector[1])
			else:
				newchar = newchar + hchar[i]
		if (debug):
			print ("hchar   :",hchar)
			print ("newchar :",newchar)
		
	hchar = newchar
	return(hchar)

#---------------------------------------------------------------------------------------------------

def move_limits (hchar, selpair, direction):
	# new string must be created because Python is not capable modify existing string
	# coordinates must be between '=' and 'h'
	
	newchar= ""
	
	for i in range(0,len(hchar)):
		if ((selpair == -2) and (i == 8)) or ((selpair == -3) and (i == 9)):
			if (selpair == -2) and (i == 8):
				x = ord(hchar[8])
				if (x + direction > ord('=')) and (x + direction < ord('h')) and (x + direction < ord(hchar[9])):
					x = x + direction
			
			if (selpair == -3) and (i == 9):
				x = ord(hchar[9])
				if (x + direction > ord('=')) and (x + direction < ord('h')) and (x + direction > ord(hchar[8])):
					x = x + direction
			
			newchar = newchar + chr(x)
		else:
			newchar = newchar + hchar[i]
	
	if (debug):
		print ("hchar   :",hchar)
		print ("newchar :",newchar)
	
	hchar = newchar
	return(hchar)

#---------------------------------------------------------------------------------------------------
# move whole glyph - all coordinates in any direction

def move_glyph (hchar, vector):
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	# create generic glyph number because glyph is no longer same as original
	newchar = "12345"
	
	# add new number of pairs + trailing spaces
	if (pairs < 10):
		newchar = newchar + ' '
	if (pairs < 100):
		newchar = newchar + ' '
	newchar = newchar + str(pairs)
	
	# status 0 until any coordinate exceeds grid
	outofgrid = 0
	
	for i in range(0,pairs):
		if (debug > 1):
			print ("pair ", i, chr(ord(hchar[2*i+8]) - vector[0]), chr(ord(hchar[2*i+9]) - vector[1]))
			
		if hchar[2*i+8] == ' ' and hchar[2*i+9] == 'R':
			# copy pen up code
			newchar = newchar + hchar[2*i+8]
			newchar = newchar + hchar[2*i+9]
		else:
			if (i == 0):
				# move left end right limit horizontally
				if (ord(hchar[2*i+8]) + vector[0] > ord('=')) and (ord(hchar[2*i+8]) + vector[0] < ord('h')):
					newchar = newchar + chr(ord(hchar[2*i+8]) + vector[0])
				else:
					outofgrid = 1
				if (ord(hchar[2*i+9]) + vector[0] > ord('=')) and (ord(hchar[2*i+9]) + vector[0] < ord('h')):
					newchar = newchar + chr(ord(hchar[2*i+9]) + vector[0])
				else:
					outofgrid = 1
			else:
				# move coordinates
				if (ord(hchar[2*i+8]) + vector[0] > ord('=')) and (ord(hchar[2*i+8]) + vector[0] < ord('h')):
					newchar = newchar + chr(ord(hchar[2*i+8]) + vector[0])
				else:
					outofgrid = 1
				if (ord(hchar[2*i+9]) + vector[1] > ord('=')) and (ord(hchar[2*i+9]) + vector[1] < ord('h')):
					newchar = newchar + chr(ord(hchar[2*i+9]) + vector[1])
				else:
					outofgrid = 1
	
	# coordinates must be between '=' and 'h'
	# if (x > 61) and (x < 104) and (y > 61) and (y < 104):
	# x = ord(hchar[pair*2+8])+vector[0]
	# y = ord(hchar[pair*2+9])+vector[1]

	if (debug):
		print ("hchar   :",hchar)
		print ("newchar :",newchar)
	
	if (outofgrid == 0):
		hchar = newchar
	return(hchar)

#---------------------------------------------------------------------------------------------------
# add new pair of coordinates

def insert_pair (hchar, selpair):
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	# new string must be created because Python is not capable modify existing string
	
	# create generic glyph number because glyph is no longer same as original
	newchar = "12345"

	# add new number of pairs + trailing spaces
	if ((pairs+1) < 10):
		newchar = newchar + ' '
	if ((pairs+1) < 100):
		newchar = newchar + ' '
	newchar = newchar + str(pairs+1)
	
	added = 0
	
	for i in range(0,pairs):
		if (i == selpair):
			# insert pair before selected pair
			newchar = newchar + hchar[2*i+8]
			newchar = newchar + hchar[2*i+9]
			newchar = newchar + hchar[2*i+8]
			newchar = newchar + hchar[2*i+9]
			added = 1
		else:
			newchar = newchar + hchar[2*i+8]
			newchar = newchar + hchar[2*i+9]
	
	if (added == 0):
		newchar = newchar + 'R'
		newchar = newchar + 'R'
		added = 1
		
	if (debug):
		print ("hchar   :",hchar)
		print ("newchar :",newchar)
	
	hchar = newchar
	
	return (hchar)

#---------------------------------------------------------------------------------------------------
# remove selected pair of coordinates

def delete_pair (hchar, selpair):
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	# new string must be created because Python is not capable modify existing string
	if (selpair > 0) and (pairs > 1) and (selpair < pairs):
		# create generic glyph number because glyph is no longer same as original
		newchar = "12345"
		
		# add new number of pairs + trailing spaces
		if ((pairs-1) < 10):
			newchar = newchar + ' '
		if ((pairs-1) < 100):
			newchar = newchar + ' '
		newchar = newchar + str(pairs-1)
		
		# first pair are limits, so technically it will be always "jumped over"
		for i in range(8,len(hchar)):
			if not ((i == selpair*2+8) or (i == selpair*2+9)):
				newchar = newchar + hchar[i]
				
		if (debug > 1):
			print ("hchar   :",hchar)
			print ("newchar :",newchar)
		
		# TODO dialog YES/NO
		hchar = newchar
	
	return (hchar)

#---------------------------------------------------------------------------------------------------

def flip_horizontal (hchar):
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	# create generic glyph number because glyph is no longer same as original
	newchar = "12345"
	
	# add new number of pairs + trailing spaces
	if (pairs < 10):
		newchar = newchar + ' '
	if (pairs < 100):
		newchar = newchar + ' '
	newchar = newchar + str(pairs)
	
	for i in range(0,pairs):
		if (debug > 1):
			print (hchar[2*i+8], ord(hchar[2*i+8]) - ord('R'), ord('R') - ord(hchar[2*i+8]),chr(ord('R') + ord('R') - ord(hchar[2*i+8])))
		if hchar[2*i+8] == ' ' and hchar[2*i+9] == 'R':
			# copy pen up code
			newchar = newchar + hchar[2*i+8]
			newchar = newchar + hchar[2*i+9]
		else:
			if (i == 0):
				# flip both coordinates for left and right limit
				newchar = newchar + chr(ord('R') + ord('R') - ord(hchar[2*i+8]))
				newchar = newchar + chr(ord('R') + ord('R') - ord(hchar[2*i+9]))
			else:
				# flip only X corrdinate
				newchar = newchar + chr(ord('R') + ord('R') - ord(hchar[2*i+8]))
				newchar = newchar + hchar[2*i+9]
	
	if (debug):
		print ("hchar   :",hchar)
		print ("newchar :",newchar)
	
	hchar = newchar
	return (hchar)

#---------------------------------------------------------------------------------------------------

def flip_vertical (hchar):
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	# create generic glyph number because glyph is no longer same as original
	newchar = "12345"
	
	# add new number of pairs + trailing spaces
	if (pairs < 10):
		newchar = newchar + ' '
	if (pairs < 100):
		newchar = newchar + ' '
	newchar = newchar + str(pairs)
	
	for i in range(0,pairs):
		if (debug > 1):
			print (hchar[2*i+9], ord(hchar[2*i+9]) - ord('R'), ord('R') - ord(hchar[2*i+9]),chr(ord('R') + ord('R') - ord(hchar[2*i+9])))
		if hchar[2*i+8] == ' ' and hchar[2*i+9] == 'R':
			# copy pen up code
			newchar = newchar + hchar[2*i+8]
			newchar = newchar + hchar[2*i+9]
		else:
			if (i == 0):
				# leave limit intact
				newchar = newchar + hchar[2*i+8]
				newchar = newchar + hchar[2*i+9]
			else:
				# flip only y corrdinate
				newchar = newchar + hchar[2*i+8]
				newchar = newchar + chr(ord('R') + ord('R') - ord(hchar[2*i+9]))
	
	if (debug):
		print ("hchar   :",hchar)
		print ("newchar :",newchar)
	
	hchar = newchar
	return (hchar)

#---------------------------------------------------------------------------------------------------
# move/swap pair of coordinates left or right

def swap_pair (hchar, direction, selpair):
	# direction can be only 'left' or 'righ'
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	# new string must be created because Python is not capable modify existing string
	
	# create generic glyph number because glyph is no longer same as original
	newchar = "12345"
	
	# add new number of pairs + trailing spaces
	if (pairs < 10):
		newchar = newchar + ' '
	if (pairs < 100):
		newchar = newchar + ' '
	newchar = newchar + str(pairs)
	
	source = 0
	target = 0
	
	if (selpair > -1):
		if (selpair == 1) and (direction == 'right'):
			source = 1
			target = 2
		
		if (selpair == (pairs - 1)) and (direction == 'left'):
			source = selpair-1
			target = selpair
		
		if (selpair > 1) and (selpair < (pairs-1)):
			if (direction == 'left'):
				source = selpair-1
				target = selpair
			else:
				source = selpair
				target = selpair+1
	
	if (source == 0) or (target == 0):
		if (debug):
			print ("cannot swap pairs")
	else:
		if (debug):
			print ("swapping "+str(source)+" - "+str(target))
		
		for i in range(0,pairs):
			if (i == source) or (i == target):
				if (i == source):
					newchar = newchar + hchar[2*target+8]
					newchar = newchar + hchar[2*target+9]
				if (i == target):
					newchar = newchar + hchar[2*source+8]
					newchar = newchar + hchar[2*source+9]
				
			else:
				newchar = newchar + hchar[2*i+8]
				newchar = newchar + hchar[2*i+9]
	
		if (debug):
			print ("hchar   :",hchar)
			print ("newchar :",newchar)
		
		hchar = newchar
	
	return (hchar)

#---------------------------------------------------------------------------------------------------
# will add second glyph to the end of first - useful for diacritics

def merge_glyphs (hchar1, hchar2):
	# create generic glyph number because glyph is no longer same as original
	newchar = "12345"
	
	if (ord(hchar1[8]) < ord(hchar2[8])):
		left_limit = hchar1[8]
	else:
		left_limit = hchar2[8]
	
	if (ord(hchar1[9]) < ord(hchar2[9])):
		right_limit = hchar2[9]
	else:
		right_limit = hchar1[9]
	
	newglyph = ""		# only pairs with coordinates
	
	pairs1 = hershey_pairs(hchar1)	# get number of pairs
	pairs2 = hershey_pairs(hchar2)	# get number of pairs
	
	pairs = 1			# number of pairs
	
	# add pairs from first glyph
	for i in range(1,pairs1):
		newglyph = newglyph + hchar1[2*i+8]
		newglyph = newglyph + hchar1[2*i+9]
		pairs = pairs + 1

	# add pen up between sequences of pairs
	newglyph = newglyph + ' '
	newglyph = newglyph + 'R'
	pairs = pairs + 1
	
	# add pairs from second glyph
	for i in range(1,pairs2):
		newglyph = newglyph + hchar2[2*i+8]
		newglyph = newglyph + hchar2[2*i+9]
		pairs = pairs + 1
		
	# add new number of pairs + trailing spaces
	if (pairs < 10):
		newchar = newchar + ' '
	if (pairs < 100):
		newchar = newchar + ' '
	newchar = newchar + str(pairs)
	
	newchar = newchar + left_limit + right_limit + newglyph
	
	if (debug):
		print ("hchar1  :",hchar1," hchar2  :",hchar2)
		print ("newchar :",newchar)
	
	return (newchar)

#---------------------------------------------------------------------------------------------------
# toggle pair between coordinates and pen up

def toggle_penup (hchar, selpair):
	pairs = hershey_pairs(hchar)	# get number of pairs
	
	# new string must be created because Python is not capable modify existing string
	
	# create generic glyph number because glyph is no longer same as original
	newchar = "12345"
	
	# add new number of pairs + trailing spaces
	if (pairs < 10):
		newchar = newchar + ' '
	if (pairs < 100):
		newchar = newchar + ' '
	newchar = newchar + str(pairs)
	
	for i in range(0,pairs):
		if (i == selpair):
			# toggle pair
			if (hchar[2*i+8] == ' ') and (hchar[2*i+9] == 'R'):
				newchar = newchar + 'R'
				newchar = newchar + 'R'
			else:
				newchar = newchar + ' '
				newchar = newchar + 'R'
		else:
			newchar = newchar + hchar[2*i+8]
			newchar = newchar + hchar[2*i+9]
	
	if (debug):
		print ("hchar   :",hchar)
		print ("newchar :",newchar)
	
	hchar = newchar
	return (hchar)

#---------------------------------------------------------------------------------------------------

def display_coordinates (position):
	x = round ((position[0] - rasterxpos)/rastersquare)
	y = round ((position[1] - rasterypos)/rastersquare)
	msg = fontsmall.render("coordinates: "+chr(x+ord('>'))+", "+chr(y+ord('>')), 1, color_text)
	screen.fill(color_background, (960, lineh*5, 160, lineh))
	screen.blit(msg, (960,lineh*5))
	
def display_penup ():
	msg = fontsmall.render("coordinates: pen up", 1, color_text)
	screen.fill(color_background, (960, lineh*5, 160, lineh))
	screen.blit(msg, (960,lineh*5))
	
def display_selected_coordinates (hchar, selpair):
	if (selpair > -1):
		if not is_penup (hchar, selpair):
			msg = fontsmall.render("coordinates: "+hchar[selpair*2+8]+", "+hchar[selpair*2+9], 1, color_text)
		else:
			msg = fontsmall.render("coordinates: pen up", 1, color_text)
		screen.fill(color_background, (960, lineh*5, 160, lineh))
		screen.blit(msg, (960,lineh*5))

#---------------------------------------------------------------------------------------------------

def select_point (hchar,position):
	x = round ((position[0] - rasterxpos)/rastersquare)
	y = round ((position[1] - rasterypos)/rastersquare)
	
	hx = chr(x+ord('>'))
	hy = chr(y+ord('>'))
	
	if (debug):
		print ('click on ['+hx+', '+hy+']')
	
	# search pair in hershey glyph
	pair = -1			# -1 until pair is found
	diameter = 4			# small circle if not pair found
	pairs = hershey_pairs(hchar)	# get number of pairs
	pairmatches = 0
	color = color_raster
	
	# pairs with coordinates
	for i in range(1,pairs):
		if (hx == hchar[2*i+8]) and (hy == hchar[2*i+9]):
			pair = i
			pairmatches = pairmatches + 1
			diameter = 6
	
	# left limit
	if (hx == hchar[8]) and (hy == 'R'):
		pair = -2
		diameter = 6
		color = color_limits
	
	# right limit
	if (hx == hchar[9]) and (hy == 'R'):
		pair = -3
		diameter = 6
		color = color_limits
	
	# draw circle on raster
	pygame.draw.circle(screen, color, (rasterxpos+x*rastersquare, rasterypos+y*rastersquare), diameter)
	
	return (pair)

#---------------------------------------------------------------------------------------------------

def main (argv):
	infilename = ''
	outfilename = ''
	
	opts, args = getopt.getopt(argv,"hi:o:",["ifile=","ofile="])
	for opt, arg in opts:
		if opt == '-h':
			print ("Simple Hershey font viewer -i <inputfile> -o <outputfile>")
			sys.exit()
		elif opt in ("-i", "--ifile"):
			infilename = arg
		elif opt in ("-o", "--ofile"):
			outfilename = arg

	if (debug):
		print ('Input file is ', infilename)
		print ('Output file is ', outfilename)
	
	if (infilename != ''):
		# alternativelly os.path.exists ?
		if (os.path.isfile (infilename)):
			# define first characters before ASCII space
			for i in range(0, 32):
				hershey_font.append (hershey_space)
			
			# open file
			infile = open(infilename, "r")

			character = 32
			
			for i in infile:
				# try ignore empty (too short) lines
				if (len(i) > 8):
					i = i.rstrip()		# remove trailing CR LF and whitespaces, there shouldn't be any
					hershey_font.append(i)
					character = character + 1
			
			print ("characters readed from file: ", (character-1))
			
			# trying detect font with ISO8859-2 extension (128 + 32 spaces)
			if (character > 159):
				# in case file did not contain all glyphs, fill missing with spaces
				if (character < 256):
					for i in range(character,256):
						hershey_font.append (hershey_space)
				maxchar = 255
				extensionset = 0
			else:
				# in case file did not contain all glyphs, fill missing with spaces
				if (character < 128):
					for i in range(character,128):
						hershey_font.append (hershey_space)
				maxchar = 127
				extensionset = 0
		else:
			print (bcol_red+"File "+infilename+" doesn't exits."+bcol_def)
			quit(-1)
	else:
		print (bcol_red+"You have to specify input file with Hershey font (jhf), use option -i or -h for help."+bcol_def)
		print (bcol_red+"To be able function, generating empty font."+bcol_def)
		
		# define empty font
		for i in range(0, 128):
			hershey_font.append (hershey_space)

		maxchar = 127
		extensionset = 0

	selpair = -1		# no pair of coordinates selected yet
	character = 65		# selected character A
	bold = 0		# standard line width
	clipboard = hershey_font[32]
	basegrid(infilename, extensionset)
	render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
	render_hershey_coordinates (hershey_font[character])
	pygame.display.flip()
	
	
	while True:
		for events in pygame.event.get():
			if events.type == pygame.VIDEOEXPOSE:
				pygame.display.flip()
				if (debug > 1):
					print ("expose")
			if events.type == pygame.KEYDOWN:
				key = pygame.key.get_pressed()
				
				if key[pygame.K_LEFT] or key[pygame.K_RIGHT] or key[pygame.K_UP] or key[pygame.K_DOWN]:
					if (selpair > -1) and (not is_penup(hershey_font[character], selpair)):
						# move point
						if key[pygame.K_LEFT]:
							if (debug > 1):
								print ("pair left")
							hershey_font[character] = move_pair (hershey_font[character], selpair, (-1,0))
						
						if key[pygame.K_RIGHT]:
							if (debug > 1):
								print ("pair right")
							hershey_font[character] = move_pair (hershey_font[character], selpair, (+1,0))
						
						if key[pygame.K_UP]:
							if (debug > 1):
								print ("pair up")
							hershey_font[character] = move_pair (hershey_font[character], selpair, (0,-1))
						
						if key[pygame.K_DOWN]:
							if (debug > 1):
								print ("pair down")
							hershey_font[character] = move_pair (hershey_font[character], selpair, (0,+1))
					
					if (selpair == -2):
						# move left limit
						if key[pygame.K_LEFT]:
							if (debug > 1):
								print ("left limit left")
							hershey_font[character] = move_limits (hershey_font[character], selpair, -1)
						
						if key[pygame.K_RIGHT]:
							if (debug > 1):
								print ("left limit right")
							hershey_font[character] = move_limits (hershey_font[character], selpair, +1)
					
					if (selpair == -3):
						# move right limit
						if key[pygame.K_LEFT]:
							if (debug > 1):
								print ("right limit left")
							hershey_font[character] = move_limits (hershey_font[character], selpair, -1)
						
						if key[pygame.K_RIGHT]:
							if (debug > 1):
								print ("right limit right")
							hershey_font[character] = move_limits (hershey_font[character], selpair, +1)
					
					# refresh screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates(hershey_font[character], selpair)
					display_selected_coordinates(hershey_font[character], selpair)
					pygame.display.flip()
				
				if key[pygame.K_KP2] or key[pygame.K_KP4] or key[pygame.K_KP6] or key[pygame.K_KP8]:
					if key[pygame.K_KP2]:
						if (debug > 1):
							print ("glyph down")
						hershey_font[character] = move_glyph (hershey_font[character], (0, +1))
					
					if key[pygame.K_KP8]:
						if (debug > 1):
							print ("glyph up")
						hershey_font[character] = move_glyph (hershey_font[character], (0, -1))
					
					if key[pygame.K_KP4]:
						if (debug > 1):
							print ("glyph left")
						hershey_font[character] = move_glyph (hershey_font[character], (-1, 0))
					
					if key[pygame.K_KP6]:
						if (debug > 1):
							print ("glyph right")
						hershey_font[character] = move_glyph (hershey_font[character], (+1, 0))
					
					# refresh screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates(hershey_font[character], selpair)
					display_selected_coordinates(hershey_font[character], selpair)
					pygame.display.flip()
				
				if key[pygame.K_n] or key[pygame.K_m] and not (key[pygame.K_LCTRL] or key[pygame.K_RCTRL]):
					if (selpair > -1):
						# move pair to the left
						if key[pygame.K_n]:
							hershey_font[character] = swap_pair (hershey_font[character], 'left', selpair)
							if (debug > 1):
								print ("swap left")
						
						if key[pygame.K_m]:
							hershey_font[character] = swap_pair (hershey_font[character], 'right', selpair)
							if (debug > 1):
								print ("swap right")
						
						# refresh screen
						screen.fill(color_background)
						basegrid(infilename, extensionset)
						render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
						render_hershey_coordinates(hershey_font[character], selpair)
						display_selected_coordinates(hershey_font[character], selpair)
						pygame.display.flip()
				
				if key[pygame.K_HOME] or key[pygame.K_END] or key[pygame.K_INSERT] or key[pygame.K_DELETE]:
					# traverse trough pairs of coordinates left or right
					pairs = hershey_pairs(hershey_font[character])	# get number of pairs
					if (selpair == -1):
						if key[pygame.K_HOME] and (pairs > 1):
							# select first pair
							selpair = 1
						if key[pygame.K_END] and (pairs > 1):
							# select last pair
							selpair = pairs - 1
					else:
						if key[pygame.K_HOME]:
							# move selection to the left
							if (selpair > 1):
								selpair = selpair-1
							else:
								selpair = pairs - 1
						if key[pygame.K_END]:
							# move selection to the right
							if (selpair < (pairs-1)):
								selpair = selpair+1
							else:
								selpair = 1
						if key[pygame.K_DELETE]:
							# remove point
							if (debug > 1):
								print ("delete pair")
							hershey_font[character] = delete_pair(hershey_font[character], selpair)
							pairs = hershey_pairs(hershey_font[character])	# get number of pairs
							# selpair must not be outside existing pairs
							if (selpair >= pairs):
								selpair = pairs-1
							# if only limits left, unselect pair
							if (pairs == 1):
								selpair = -1
							
					if key[pygame.K_INSERT]:
						if (debug > 1):
							print ("insert pair")
						hershey_font[character] = insert_pair(hershey_font[character], selpair)
					
					# refresh screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates(hershey_font[character], selpair)
					display_selected_coordinates(hershey_font[character], selpair)
					pygame.display.flip()
				
				if key[pygame.K_p]:
					# toggle selected pair between coordinates and pen up
					if (selpair > -1):
						if (debug):
							print ("toggle pen up")
						hershey_font[character] = toggle_penup(hershey_font[character], selpair)
						
						# refresh screen
						screen.fill(color_background)
						basegrid(infilename, extensionset)
						render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
						render_hershey_coordinates(hershey_font[character], selpair)
						display_selected_coordinates(hershey_font[character], selpair)
						pygame.display.flip()
					
				if key[pygame.K_a]:
					# display glyph selection (all ASCII characters)
					selpair = -1		# unselect pair of coordinates
					r = glyph_selection (extensionset, maxchar)	# returns ASCII code or -1 if nothing selected
					if (r > 0):
						character = r
						if not (maxchar == 127):
							if (r < 128):
								extensionset = 0
							else:
								extensionset = 1
					
					# refresh screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates (hershey_font[character])
					pygame.display.flip()
				
				if key[pygame.K_e]:
					if key[pygame.K_LCTRL] or key[pygame.K_RCTRL]:
						# export whole font ASCII code 32 to 127
						if (debug > 1):
							print ("CTRL+E = export whole font")
						for i in range (32,128):
							export_character(i, extensionset, maxchar)
					else:
						# export one glyph
						if (debug > 1):
							print ("E = export glyph "+str(character))
						export_character(character, extensionset, maxchar)
				
				if (key[pygame.K_LCTRL] or key[pygame.K_RCTRL]) and key[pygame.K_s]:
					# TODO real save with file dialog
					if (debug > 1):
						print ("CTRL+S = export whole font")
					print (bcol_yellow+"----------"+bcol_def)
					for i in range (32,128):
						print (hershey_font[i])
					if (maxchar == 255):
						for i in range (128,256):
							print (hershey_font[i])
					print (bcol_yellow+"----------"+bcol_def)
				
				if key[pygame.K_F1]:
					# simple help
					selpair = -1		# unselect pair of coordinates
					helppage()
					
					# refresh screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates (hershey_font[character])
					pygame.display.flip()
				
				if key[pygame.K_r] or key[pygame.K_F5]:
					selpair = -1		# unselect pair of coordinates
					# update screen (slow for key R)
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					if key[pygame.K_r]:
						delay = 1
						pygame.display.flip()
					else:
						delay = 0
					render_hershey_glyph(hershey_font[character], character, delay, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates (hershey_font[character])
					pygame.display.flip()
				
				if (key[pygame.K_h] or key[pygame.K_v]) and not (key[pygame.K_LCTRL] or key[pygame.K_RCTRL]):
					if key[pygame.K_h]:
						# horizontal flip
						if (debug > 1):
							print ("key H")
						hershey_font[character] = flip_horizontal(hershey_font[character])
					if key[pygame.K_v]:
						# vertical flip
						if (debug > 1):
							print ("key V")
						hershey_font[character] = flip_vertical(hershey_font[character])
					
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates(hershey_font[character], selpair)
					display_selected_coordinates(hershey_font[character], selpair)
					pygame.display.flip()
				
				if key[pygame.K_i]:
					# info about font - additional statistics
					hershey_statistics()
					pygame.display.flip()
				
				if key[pygame.K_b]:
					# extra bold
					if bold < 2:
						bold = bold + 1
					else:
						bold = 0
					
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates (hershey_font[character])
					pygame.display.flip()
				
				if key[pygame.K_PLUS] or key[pygame.K_KP_PLUS] or key[pygame.K_PAGEDOWN]:
					selpair = -1		# unselect pair of coordinates
					
					if (maxchar == 127):
						if (character < 127):
							character = character+1
					else:
						if (character < 255):
							character = character+1
						if (character == 128):
							character = 160
							extensionset = 1
					
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates (hershey_font[character])
					pygame.display.flip()
				
				if key[pygame.K_MINUS] or key[pygame.K_KP_MINUS] or key[pygame.K_PAGEUP]:
					selpair = -1		# unselect pair of coordinates
					if (character > 32):
						character = character-1
						
						if (character == 159):
							character = 127
							extensionset = 0
						
						# update screen
						screen.fill(color_background)
						basegrid(infilename, extensionset)
						render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
						render_hershey_coordinates (hershey_font[character])
						pygame.display.flip()
				
				if (key[pygame.K_LCTRL] or key[pygame.K_RCTRL]) and key[pygame.K_c]:
					if (debug):
						print ("copy to clipboard")
					clipboard = hershey_font[character]
				
				if (key[pygame.K_LCTRL] or key[pygame.K_RCTRL]) and key[pygame.K_v]:
					if (debug):
						print ("paste from clipboard")
					if (len(clipboard) > 8):
						hershey_font[character] = clipboard
						
						selpair = -1		# unselect pair of coordinates
						
						# update screen
						screen.fill(color_background)
						basegrid(infilename, extensionset)
						render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
						render_hershey_coordinates (hershey_font[character])
						pygame.display.flip()
				
				if (key[pygame.K_LCTRL] or key[pygame.K_RCTRL]) and key[pygame.K_m]:
					if (debug):
						print ("merge from clipboard")
					hershey_font[character] = merge_glyphs(hershey_font[character], clipboard)
					
					selpair = -1		# unselect pair of coordinates
					
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates (hershey_font[character])
					pygame.display.flip()
					
				if not key[pygame.K_LCTRL] and not key[pygame.K_RCTRL] and key[pygame.K_x]:
					if (extensionset == 0):
						if (maxchar == 127):
							if (debug):
								print ("creating extension set")
							for i in range(character,256):
								hershey_font.append (hershey_space)
							maxchar = 255
							extensionset = 1
							if (character < 128):
								character = character + 128
						else:
							if (debug):
								print ("switching to extension set")
							extensionset = 1
							if (character < 128):
								character = character + 128
					else:
						if (debug):
							print ("switching to standard set")
						extensionset = 0
						if (character > 127):
							character = character - 128
						if (character < 32):
							character = 32
					
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates (hershey_font[character])
					pygame.display.flip()
					
					
			#if events.type == pygame.MOUSEMOTION:
				# pos = pygame.mouse.get_pos()
			
			if events.type == pygame.MOUSEBUTTONDOWN:
				pos = pygame.mouse.get_pos()
				maxx = rasterxpos+rastersquare*41
				maxy = rasterypos+rastersquare*41
				
				# base grid
				if (pos[0] > rasterxpos and pos[0] < maxx and pos[1] > rasterypos and pos[1] < maxy):
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					selpair = select_point(hershey_font[character], pos)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates(hershey_font[character], selpair)
					display_coordinates(pos)
					pygame.display.flip()
				
				# coordinates at right side of base grid - horizontal guides
				if (pos[0] > maxx and pos[0] < (maxx+rastersquare) and pos[1] > rasterypos and pos[1] < maxy):
					y = round((pos[1] - rasterypos)/rastersquare)
					if (guidelny[y] == 0):
						guidelny[y] = 1
					else:
						guidelny[y] = 0
					
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates(hershey_font[character], selpair)
					display_coordinates(pos)
					pygame.display.flip()
				
				# coordinates bellow base grid - vertical guides
				if (pos[0] > rasterxpos and pos[0] < maxx and pos[1] > maxy and pos[1] < (maxy+rastersquare)):
					x = round((pos[0] - rasterxpos)/rastersquare)
					if (guidelnx[x] == 0):
						guidelnx[x] = 1
					else:
						guidelnx[x] = 0
					
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates(hershey_font[character], selpair)
					display_coordinates(pos)
					pygame.display.flip()
				
				# hershey coordinates area bellow base grid
				if (pos[1] > (rasterypos+rastersquare*42+10)):
					selpair = click_on_hershey_coordinates(hershey_font[character], pos)
					if (selpair > -1):
						if (debug):
							print ("selpair",selpair);
					else:
						if (debug):
							print ("unselecting pair");
							
					# update screen
					screen.fill(color_background)
					basegrid(infilename, extensionset)
					render_hershey_glyph(hershey_font[character], character, 0, bold, selpair, extensionset, maxchar)
					render_hershey_coordinates(hershey_font[character], selpair)
					display_selected_coordinates(hershey_font[character], selpair)
					pygame.display.flip()

			#if events.type == pygame.MOUSEBUTTONUP:
			if events.type == pygame.QUIT:
				quit(0)
		pygame.time.wait(2)

if __name__ == "__main__":
	main(sys.argv[1:])

main ()
